# -*- coding: UTF-8 -*-

import json
import time
from pathlib import Path
from urllib.parse import quote

import pandas as pd
import requests
from huaweicloudsdkcore.auth.credentials import BasicCredentials
from huaweicloudsdkcore.exceptions import exceptions
from huaweicloudsdkocr.v1 import *
from huaweicloudsdkocr.v1.region.ocr_region import OcrRegion
from loguru import logger
from pofile import get_files
from poprogress import simple_progress

from poocr.lib.CommonUtils import img2base64, pdf2base64
from poocr.lib.apig_sdk import signer


def household2excel(ak, sk, img_path: list, output_excel: str = r'./household2excel.xlsx'):
    credentials = BasicCredentials(ak, sk)

    client = OcrClient.new_builder() \
        .with_credentials(credentials) \
        .with_region(OcrRegion.value_of("cn-north-4")) \
        .build()

    request = RecognizeHouseholdRegisterRequest()

    ticket_list = []
    img_list=[]
    for path in img_path:
        img_list.extend( get_files(path))
    for item in img_list:
        request.body = HouseholdRegisterRequestBody(
            image=img2base64(item)
        )
        try:
            response = client.recognize_household_register(request)
            time.sleep(3)
            # response.result

            if response.status_code == 200:
                response_dict = response.to_dict()
                for res_info in simple_progress(response_dict["result"]):
                    if res_info['type'] == "登记页":
                        one_content = res_info['content']
                        one_content['文件名'] = Path(item).stem
                        if len(one_content['id_card_number']) < 18:
                            one_content['备注'] = '身份证号码小于18位'
                        ticket_list.append(one_content)
        except exceptions.ClientRequestException as e:
            logger.info(f'{Path(item).stem}识别失败，原因：{e.error_msg}')
    if len(ticket_list) > 0:
        ticket_df = pd.DataFrame(ticket_list)
        ticket_df.to_excel(output_excel, index=None)
    else:
        logger.info('没有识别到任何户口本图片')


def BankReceipt2excel(ak, sk, file_path, output_excel: str = r'./BankReceipt2excel.xlsx'):
    credentials = BasicCredentials(ak, sk)

    client = OcrClient.new_builder() \
        .with_credentials(credentials) \
        .with_region(OcrRegion.value_of("cn-north-4")) \
        .build()

    # request = RecognizeHouseholdRegisterRequest()
    request = RecognizeBankReceiptRequest()

    ticket_list = []
    img_list = get_files(file_path)
    for item in img_list:
        for base64_encoded_pdf in pdf2base64(item):
            request.body = BankReceiptRequestBody(
                data=base64_encoded_pdf
            )
            try:
                response = client.recognize_bank_receipt(request)
                time.sleep(3)
                if response.status_code == 200:
                    response_dict = response.to_dict()
                    for bank_info in simple_progress(response_dict["result"]["bank_receipt_list"]):
                        one_content = bank_info['kv_pair_list']
                        one_content_dict = {}
                        for kv in one_content:
                            one_content_dict[kv['key']] = kv['value']
                        ticket_list.append(one_content_dict)
            except exceptions.ClientRequestException as e:
                logger.info(f'{Path(item).stem}识别失败，原因：{e.error_msg}')
    if len(ticket_list) > 0:
        ticket_df = pd.DataFrame(ticket_list)
        ticket_df.to_excel(output_excel, index=None)
    else:
        logger.info('没有识别到任何银行回单PDF/图片')


def household2excel2(ak, sk, img_path: list, output_excel: str = r'./household2excel.xlsx'):
    sig = signer.Signer()
    # 认证用的ak和sk硬编码到代码中或者明文存储都有很大的安全风险，建议在配置文件或者环境变量中密文存放，使用时解密，确保安全；
    # 本示例以ak和sk保存在环境变量中为例，运行本示例前请先在本地环境中设置环境变量HUAWEICLOUD_SDK_AK和HUAWEICLOUD_SDK_SK。
    sig.Key = ak
    sig.Secret = sk
    res_list = []
    img_list = []
    for path in img_path:
        img_list.extend(get_files(path))
    for one_img in img_list:
        img = img2base64(one_img)
        body = "base64=data:image/jpeg;base64," + quote(img)
        r = signer.HttpRequest(method="POST",
                               url="https://jmhkbocr.apistore.huaweicloud.com/ocr/householdRegister",
                               headers={'Content-Type': 'application/x-www-form-urlencoded'},
                               body=body)
        sig.Sign(r)
        resp = requests.request(r.method, r.scheme + "://" + r.host + r.uri, headers=r.headers, data=r.body)
        content = resp.content
        api_res_json = json.loads(content.decode('utf-8'))
        if api_res_json['code'] == 200 and api_res_json['data'] != None:
            api_res_json['data']['文件名'] = Path(one_img).stem
            res_list.append(api_res_json['data'])
    res_df = pd.DataFrame(res_list)
    # res_df.append_(pd.DataFrame(, index=[0]))
    res_df.to_excel(output_excel, index=None)


if __name__ == '__main__':
    pass



# -*- coding: UTF-8 -*-

import toml
from loguru import logger

from poocr.lib.Const import DEFAULT_CONFIG_PATH_NAME, DEFAULT_CONFIG_NAME, TUTORIA_VIDEO


class poocrConfig():
    """
    通过配置文件来验证id和key，现在已经弃用。改为通过函数里传参：id，key
    """

    def __init__(self):
        self.config_info = None

    def get_config(self, configPath):
        """
        解析配置文件
        :param toml_path: 存放配置文件的位置和名称，在py文件的同级路径下
        :return: 加载后的信息
        """
        try:
            if configPath == None:
                self.config_info = toml.load(DEFAULT_CONFIG_PATH_NAME)
            else:
                self.config_info = toml.load(configPath)
            logger.info(f'配置文件【{DEFAULT_CONFIG_NAME}】读取成功')
            return self.config_info
        except:
            logger.info(f'配置文件【{DEFAULT_CONFIG_NAME}】读取失败，请查看视频，进行配置：{TUTORIA_VIDEO}')



import os
import unittest

from poocr.api.ocr2excel import *


class TestTencent(unittest.TestCase):

    def setUp(self):
        self.SecretId = os.getenv("SecretId", None)
        self.SecretKey = os.getenv("SecretKey", None)

        self.ak = os.getenv('ak', None)
        self.sk = os.getenv('sk', None)

    def test_id_key(self):
        if not (self.SecretId and self.SecretKey):
            raise Exception("请配置腾讯云的SecretId和SecretKey")
        if not (self.ak and self.sk):
            raise Exception("请配置华为云的ak和sk")
        logger.info(f'腾讯云 {self.SecretId} {self.SecretKey}')
        logger.info(f'华为云 {self.ak} {self.sk}')

    def test_version(self):
        logger.info(f"poocr version:{poocr.__version__}")

    def test_doc(self):
        logger.info(f"poocr doc:{poocr.__doc__}")



from poocr.api import ocr
from poocr.api import ocr2excel

__doc__ = """
【poocr库】，功能持续更新中
使用有问题 or 提交你的功能需求 or 参与项目开发
1、全部功能【视频 & 文字】教程：https://www.python-office.com/course-002/5-poocr/5-poocr.html
2、请+【项目交流群】：http://www.python4office.cn/wechat-group/
3、本开源项目的【代码仓库】：https://gitcode.com/CoderWanFeng1/poocr
4、给小白的【50讲Python自动化办公】：https://www.python-office.com/course/50-python-office.html
"""

__version__ = "1.0.0"



# -*- coding: UTF-8 -*-


import pymupdf  # PyMuPDF


def get_error_info(error_info):
    """
    检查接口返回数据是否错误
    :param error_info: 调用接口的返回数据
    :return:
    """
    error_url = 'http://python4office.cn/pobaidu/pobaidu-error/'
    if error_info.get('error_code', False):
        return f"接口调用错误，错误信息是{error_info}，原因和解决方法请查看官方文档：{error_url}"
    return False


def img2base64(imgPath):
    with open(imgPath, "rb") as f:
        data = f.read()
        encodestr = base64.b64encode(data)  # 得到 byte 编码的数据
        picbase = str(encodestr, 'utf-8')
        return picbase


import base64


def pdf2base64(pdf_path):
    base64_encoded_pdf = []
    pdf = pymupdf.open(pdf_path)
    for i in range(len(pdf)):
        pdf_bytes = pdf.convert_to_pdf(i, i + 1)
        # 灏嗗浘鐗囪浆鎹负Base64缂栫爜鐨勫瓧绗︿覆
        base64_encoded_pdf.append(base64.b64encode(pdf_bytes).decode('utf-8'))
    # 鍏抽棴PDF鏂囨。
    pdf.close()
    return base64_encoded_pdf


if __name__ == '__main__':
    pdf2base64(pdf_path=r'D:\workplace\code\github\poocr\tests\test_files\4-银行回单\huawei\002.pdf')



import base64

from loguru import logger
from tencentcloud.common import credential
from tencentcloud.common.exception.tencent_cloud_sdk_exception import TencentCloudSDKException
from tencentcloud.common.profile.client_profile import ClientProfile
from tencentcloud.common.profile.http_profile import HttpProfile
from tencentcloud.ocr.v20181119 import ocr_client, models

from poocr.lib.Config import poocrConfig
from poocr.lib.Const import NO_FILE_ERROR


class OfdOCR(poocrConfig):
    def __init__(self):
        self.TENCENT_AI_CFG = None
        self.TENCENTCLOUD_SECRET_ID = None
        self.TENCENTCLOUD_SECRET_KEY = None
        self.CLIENT = None

    def set_config(self, id, key):
        """
        加载配置信息，生成client，获取配置信息：
        :param configPath: 可以自定义config文件的名称和位置，有默认值
        :return:
        """
        if id != None and key != None:
            self.TENCENTCLOUD_SECRET_ID = id
            self.TENCENTCLOUD_SECRET_KEY = key
        else:
            logger.error("请先设置腾讯云的SecretId和SecretKey")
        cred = credential.Credential(self.TENCENTCLOUD_SECRET_ID, self.TENCENTCLOUD_SECRET_KEY)
        httpProfile = HttpProfile()
        httpProfile.endpoint = "ocr.tencentcloudapi.com"
        clientProfile = ClientProfile()
        clientProfile.httpProfile = httpProfile
        self.client = ocr_client.OcrClient(cred, "ap-beijing", clientProfile)
        return self.client

    def get_params(self, OfdFileBase64, OfdFileUrl):
        """
        图片的OfdFileUrl、OfdFileBase64必须提供一个，如果都提供，只使用OfdFileUrl。
        :param OfdFileBase64:
        :param OfdFileUrl:
        :return:
        """
        if OfdFileUrl:
            params = '{\"OfdFileUrl\":\"%s\"} ' % OfdFileUrl
        elif OfdFileBase64:
            params = '{\"OfdFileBase64\":\"%s\"} ' % OfdFileBase64
        else:
            return NO_FILE_ERROR
        return params

    def DoOCR(self, OfdFileBase64=None, OfdFileUrl=None):

        try:
            req = models.VerifyOfdVatInvoiceOCRRequest()
            req.from_json_string(self.get_params(OfdFileBase64, OfdFileUrl))
            resp = self.client.VerifyOfdVatInvoiceOCR(req)
            return resp

        except TencentCloudSDKException as err:
            logger.info(err)


def VerifyOfdVatInvoiceOCR(ofd_file_path=None, ofd_file_url=None, id=None, key=None):
    ocr = OfdOCR()
    ocr.set_config(id, key)
    if ofd_file_path:
        with open(ofd_file_path, 'rb') as file:
            pdf_data = file.read()
            base64_encoded_pdf = base64.b64encode(pdf_data).decode('utf-8')
            ocr_res = ocr.DoOCR(OfdFileBase64=base64_encoded_pdf)
    elif ofd_file_url:
        ocr_res = ocr.DoOCR(OfdFileUrl=ofd_file_url)

    return ocr_res


if __name__ == '__main__':
    SecretId = os.getenv("SecretId", None)
    SecretKey = 'xxx'
    ofd_file_path = r'./suzhou.ofd'
    res = VerifyOfdVatInvoiceOCR(ofd_file_path=ofd_file_path, id=SecretId, key=SecretKey)
    logger.error(res)
    ofd_file_url = 'https://test-1300615378.cos.ap-nanjing.myqcloud.com/suzhou.ofd'
    res = VerifyOfdVatInvoiceOCR(ofd_file_url=ofd_file_url, id=SecretId, key=SecretKey)
    logger.error(res)



import os
import unittest

from poocr.api.ocr2excel import *


class Ocr2Excel(unittest.TestCase):
    """
    test for ocr2excel.py
    """

    def setUp(self):
        # 腾讯
        # self.SecretId = os.getenv("SecretId", None)
        # self.SecretKey = os.getenv("SecretKey", None)
        self.SecretId = os.getenv("SecretId", None)
        self.SecretKey = os.getenv("SecretKey", None)

    def test_RecognizeGeneralInvoiceOCR2Excel(self):
        RecognizeGeneralInvoiceOCR2Excel(input_path='../test_files/RecognizeGeneralInvoiceOCR/样例.png',
                                         output_path='../test_files/RecognizeGeneralInvoiceOCR',
                                         id=self.SecretId,
                                         key=self.SecretKey, sub_type=2)
        self.assertTrue(
            os.path.exists("../test_files/RecognizeGeneralInvoiceOCR/RecognizeGeneralInvoiceOCR2Excel.xlsx"))
        os.remove('../test_files/RecognizeGeneralInvoiceOCR/RecognizeGeneralInvoiceOCR2Excel.xlsx')

    def test_VatInvoiceOCR2Excel(self):
        VatInvoiceOCR2Excel(input_path='../test_files/VatInvoiceOCR',
                            output_path=r'../test_files/VatInvoiceOCR',
                            id=self.SecretId,
                            key=self.SecretKey)
        self.assertTrue(
            os.path.exists("../test_files/VatInvoiceOCR/VatInvoiceOCR2Excel.xlsx"))
        os.remove('../test_files/VatInvoiceOCR/VatInvoiceOCR2Excel.xlsx')

    def test_BankCardOCR2Excel(self):
        BankCardOCR2Excel(input_path='../test_files/BankCard/bankcard.png',
                          output_path=r'../test_files/BankCard',
                          id=self.SecretId,
                          key=self.SecretKey)
        self.assertTrue(
            os.path.exists("../test_files/BankCard/BankCardOCR2Excel.xlsx"))
        os.remove('../test_files/BankCard/BankCardOCR2Excel.xlsx')

    def test_BankSlipOCR2Excel(self):
        BankSlipOCR2Excel(input_path='../test_files/BankSlip/bankslip.png',
                          output_path=r'../test_files/BankSlip',
                          id=self.SecretId,
                          key=self.SecretKey)
        self.assertTrue(
            os.path.exists("../test_files/BankSlip/BankSlipOCR2Excel.xlsx"))
        os.remove('../test_files/BankSlip/BankSlipOCR2Excel.xlsx')

    def test_IDCardOCRFront2Excel(self):
        IDCardOCR2Excel(input_path='../test_files/IDCard/idcard_front.png',
                        output_path=r'../test_files/IDCard',
                        id=self.SecretId,
                        key=self.SecretKey)
        self.assertTrue(
            os.path.exists("../test_files/IDCard/IDCardOCR2Excel.xlsx"))
        os.remove('../test_files/IDCard/IDCardOCR2Excel.xlsx')

    def test_IDCardOCRBack2Excel(self):
        IDCardOCR2Excel(input_path='../test_files/IDCard/idcard_bak.png',
                        output_path=r'../test_files/IDCard',
                        id=self.SecretId,
                        key=self.SecretKey)
        self.assertTrue(
            os.path.exists("../test_files/IDCard/IDCardOCR2Excel.xlsx"))
        os.remove('../test_files/IDCard/IDCardOCR2Excel.xlsx')

    def test_TrainTicketOCR2Excel(self):
        TrainTicketOCR2Excel(input_path=r'../test_files/TrainTicket/img.png',
                             output_excel=r'../test_files/TrainTicket/ticket.xlsx',
                             configPath=r'../test_files/poocr-config.toml',
                             id=self.SecretId,
                             key=self.SecretKey)
        self.assertTrue(
            os.path.exists("../test_files/TrainTicket/ticket.xlsx"))
        os.remove('../test_files/TrainTicket/ticket.xlsx')

    def test_LicensePlateOCR2Excel(self):
        LicensePlateOCR2Excel(input_path=r'../test_files/LicensePlate/img.png',
                              output_excel=r'../test_files/LicensePlate/LicensePlate.xlsx',
                              configPath=r'../test_files/poocr-config.toml',
                              id=self.SecretId,
                              key=self.SecretKey)
        self.assertTrue(
            os.path.exists("../test_files/LicensePlate/LicensePlate.xlsx"))
        os.remove('../test_files/LicensePlate/LicensePlate.xlsx')

    def test_BizLicenseOCR2Excel(self):
        BizLicenseOCR2Excel(input_path=r'../test_files/BizLicense/img.png',
                            output_excel=r'../test_files/BizLicense/BizLicense.xlsx',
                            configPath=r'../test_files/poocr-config.toml',
                            id=self.SecretId,
                            key=self.SecretKey)
        self.assertTrue(
            os.path.exists("../test_files/BizLicense/BizLicense.xlsx"))
        os.remove('../test_files/BizLicense/BizLicense.xlsx')

    def test_household2excel(self):
        household2excel(ak=self.SecretId, sk=self.SecretKey, img_path=['../test_files/Household'],
                        output_excel=r'../test_files/Household/household2excel.xlsx')
        # household2excel(ak=r'HPUAKZ48UYAHP4HZ7NAS', sk=r'0hLDibc6ezOIOohNSnuLDbBO3Ue1U6NS4PIkmyur',img_path=['../test_files/Household'],
        #                      output_excel=r'../test_files/Household/household2excel.xlsx')
        self.assertTrue(
            os.path.exists("../test_files/Household/household2excel.xlsx"))
        os.remove('../test_files/Household/household2excel.xlsx')

    def test_household2excel2(self):
        household2excel2(ak=self.SecretId, sk=self.SecretKey, img_path=['../test_files/Household'],
                         output_excel=r'../test_files/Household/household2excel.xlsx')
        self.assertTrue(
            os.path.exists("../test_files/Household/household2excel.xlsx"))
        os.remove('../test_files/Household/household2excel.xlsx')



# -*- coding: UTF-8 -*-
'''
@学习网站      ：https://www.python-office.com
@读者群     ：http://www.python4office.cn/wechat-group/
@作者  ：B站/抖音/微博/小红书/公众号，都叫：程序员晚枫，微信：CoderWanFeng
@代码日期    ：2025/3/10 20:59 
@本段代码的视频说明     ：
'''



# -*- coding: UTF-8 -*-

import json

from loguru import logger
from tencentcloud.common import credential
from tencentcloud.common.exception.tencent_cloud_sdk_exception import TencentCloudSDKException
from tencentcloud.common.profile.client_profile import ClientProfile
from tencentcloud.common.profile.http_profile import HttpProfile
from tencentcloud.ocr.v20181119 import ocr_client, models

from poocr.lib.Config import poocrConfig
from poocr.lib.Const import NO_FILE_ERROR


class OCR(poocrConfig):
    def __init__(self):
        self.TENCENT_AI_CFG = None
        self.TENCENTCLOUD_SECRET_ID = None
        self.TENCENTCLOUD_SECRET_KEY = None
        self.CLIENT = None

    def set_config(self, configPath, id, key):
        """
        加载配置信息，生成client，获取配置信息：
        :param configPath: 可以自定义config文件的名称和位置，有默认值
        :return:
        """
        if id != None and key != None:
            self.TENCENTCLOUD_SECRET_ID = id
            self.TENCENTCLOUD_SECRET_KEY = key
        else:
            self.TENCENT_AI_CFG = self.get_config(configPath)
            if self.TENCENT_AI_CFG['tencent-ai']['TENCENTCLOUD_SECRET_ID'] and self.TENCENT_AI_CFG['tencent-ai'][
                'TENCENTCLOUD_SECRET_KEY']:
                self.TENCENTCLOUD_SECRET_ID = self.TENCENT_AI_CFG['tencent-ai']['TENCENTCLOUD_SECRET_ID']
                self.TENCENTCLOUD_SECRET_KEY = self.TENCENT_AI_CFG['tencent-ai']['TENCENTCLOUD_SECRET_KEY']
        cred = credential.Credential(self.TENCENTCLOUD_SECRET_ID, self.TENCENTCLOUD_SECRET_KEY)
        httpProfile = HttpProfile()
        httpProfile.endpoint = "ocr.tencentcloudapi.com"
        clientProfile = ClientProfile()
        clientProfile.httpProfile = httpProfile
        self.client = ocr_client.OcrClient(cred, "ap-beijing", clientProfile)
        return self.client

    def get_params(self, ImageBase64, ImageUrl, is_pdf=False):
        """
        图片的ImageUrl、ImageBase64必须提供一个，如果都提供，只使用ImageUrl。
        如果 type 不为 None，则在参数中添加 type 字段。
        :param ImageBase64: 图片的 Base64 编码
        :param ImageUrl: 图片的 URL
        :param type: 类型列表，列表中的每个元素都是 int 类型
        :return: 包含图片信息和类型信息的 JSON 字符串或错误信息
        """
        if ImageUrl:
            params = {"ImageUrl": ImageUrl}
        elif ImageBase64:
            params = {"ImageBase64": ImageBase64}
        else:
            return json.dumps(NO_FILE_ERROR)

        if is_pdf:
            params['EnableMultiplePage'] = True
        return json.dumps(params)

    def DoOCR(self, OCR_NAME, ImageBase64, ImageUrl, IsPdf=False):

        try:
            ocr_req_func = getattr(models, f'{OCR_NAME}Request', None)
            req = ocr_req_func()
            req.from_json_string(self.get_params(ImageBase64, ImageUrl, (OCR_NAME == 'RecognizeGeneralInvoice' and IsPdf)))
            ocr_func = getattr(self.client, f'{OCR_NAME}', None)
            if OCR_NAME == 'VatInvoiceOCR' or OCR_NAME == 'BankSlipOCR':
                req.IsPdf = IsPdf
            resp = ocr_func(req)
            return resp

        except TencentCloudSDKException as err:
            logger.info(err)

    def VatInvoiceOCR(self, ImageBase64, ImageUrl):
        """
        本接口支持增值税专用发票、增值税普通发票、增值税电子发票全字段的内容检测和识别，
        包括发票代码、发票号码、打印发票代码、打印发票号码、开票日期、合计金额、校验码、税率、合计税额、价税合计、购买方识别号、复核、销售方识别号、开票人、密码区1、密码区2、密码区3、密码区4、发票名称、购买方名称、销售方名称、服务名称、备注、规格型号、数量、单价、金额、税额、收款人等字段。

        默认接口请求频率限制：10次/秒。
        图片的 ImageUrl、ImageBase64 必须提供一个，如果都提供，只使用 ImageUrl。
        :param ImageBase64:
        :param ImageUrl:
        :return:
        """
        try:
            req = models.VatInvoiceOCRRequest()
            req.from_json_string(self.get_params(ImageBase64, ImageUrl))
            resp = self.client.VatInvoiceOCR(req)
            return resp

        except TencentCloudSDKException as err:
            print(err)



import os
import unittest

from loguru import logger

from poocr.api.ocr import IDCardOCR, PassportOCR, VatInvoiceOCR


class TestOCR(unittest.TestCase):
    """
    ocr.py测试用的代码
    """

    def setUp(self):
        self.SecretId = os.getenv("SecretId", None)
        self.SecretKey = os.getenv("SecretKey", None)

        self.ak = os.getenv('ak', None)
        self.sk = os.getenv('sk', None)

    def test_IDCardOCR(self):
        r = IDCardOCR(img_path=r'../test_files/IDCard/idcard_bak.png', id=self.SecretId, key=self.SecretKey)
        logger.info(r)

    def test_PassportOCR(self):
        r = PassportOCR(
            img_path=r'.\passport.png',
            id=self.SecretId, key=self.SecretKey)
        logger.info(r)

    def test_VatInvoiceOCR(self):
        v_r = VatInvoiceOCR(img_path=r'../test_files/VatInvoiceOCR/img.png', id=self.SecretId, key=self.SecretKey)
        logger.info(v_r)



# -*- coding: UTF-8 -*-

import base64
from pathlib import Path

import potx
import pymupdf

from poocr.core.BaiduOCR import BaiduOCR
from poocr.core.OCR import OCR
from poocr.lib.CommonUtils import img2base64


def get_ocr(configPath, id, key):
    ocr = OCR()
    ocr.set_config(configPath, id, key)
    return ocr


def do_api(OCR_NAME, img_path, img_url, configPath, id, key, pdf_path=None):
    """
    通过类的方法名，直接调用方法
    :return:
    """
    ocr = get_ocr(configPath, id, key)
    is_pdf = False
    if Path(img_path).suffix == '.pdf':
        is_pdf = True
    if pdf_path:
        # 打开PDF文件
        pdf = pymupdf.open(pdf_path)

        # 存储所有页面的识别结果  1
        all_results = []

        # 遍历每一页  2
        for page_num in range(len(pdf)):
            pdf_bytes = pdf.convert_to_pdf(page_num, page_num + 1)
            # 将图片转换为Base64编码的字符串
            base64_encoded_pdf = base64.b64encode(pdf_bytes).decode('utf-8')

            # 识别当前页  3
            page_result = ocr.DoOCR(OCR_NAME, ImageBase64=base64_encoded_pdf, ImageUrl=img_url, IsPdf=True)

            # 将当前页结果添加到总结果中  4
            all_results.append(page_result)
        # 关闭PDF文档
        pdf.close()
        # ocr_res = ocr.DoOCR(OCR_NAME, ImageBase64=base64_encoded_pdf, ImageUrl=img_url, IsPdf=True)
        # 如果只有一页，直接返回结果 6
        if len(all_results) == 1:
            return all_results[0]
        # 否则返回所有页面的结果列表
        return all_results
        # pdf_data = file.read()
        # base64_encoded_pdf = base64.b64encode(pdf_data).decode('utf-8')
        # ocr_res = ocr.DoOCR(OCR_NAME, ImageBase64=base64_encoded_pdf, ImageUrl=img_url, IsPdf=True)
    elif img_url:
        ocr_res = ocr.DoOCR(OCR_NAME, ImageBase64=img_path, ImageUrl=img_url, IsPdf=is_pdf)
    else:
        ImageBase64 = img2base64(img_path)
        ocr_res = ocr.DoOCR(OCR_NAME, ImageBase64=ImageBase64, ImageUrl=img_url, IsPdf=is_pdf)
    return ocr_res


def AdvertiseOCR(img_path=None, img_url=None, configPath=None, id=None, key=None):
    return potx.ocr.AdvertiseOCR(img_path, img_url, configPath, id, key)


def ArithmeticOCR(img_path=None, img_url=None, configPath=None, id=None, key=None):
    return potx.ocr.ArithmeticOCR(img_path, img_url, configPath, id, key)


def BankCardOCR(img_path=None, img_url=None, configPath=None, id=None, key=None):
    return potx.ocr.BankCardOCR(img_path, img_url, configPath, id, key)


def BizLicenseOCR(img_path=None, img_url=None, configPath=None, id=None, key=None):
    return potx.ocr.BizLicenseOCR(img_path, img_url, configPath, id, key)


def BusInvoiceOCR(img_path=None, img_url=None, configPath=None, id=None, key=None):
    return potx.ocr.BusInvoiceOCR(img_path, img_url, configPath, id, key)


def BusinessCardOCR(img_path=None, img_url=None, configPath=None, id=None, key=None):
    return potx.ocr.BusinessCardOCR(img_path, img_url, configPath, id, key)


def CarInvoiceOCR(img_path=None, img_url=None, configPath=None, id=None, key=None):
    return potx.ocr.CarInvoiceOCR(img_path, img_url, configPath, id, key)


def ClassifyDetectOCR(img_path=None, img_url=None, configPath=None, id=None, key=None):
    return potx.ocr.ClassifyDetectOCR(img_path, img_url, configPath, id, key)


def DriverLicenseOCR(img_path=None, img_url=None, configPath=None, id=None, key=None):
    return potx.ocr.DriverLicenseOCR(img_path, img_url, configPath, id, key)


def DutyPaidProofOCR(img_path=None, img_url=None, configPath=None, id=None, key=None):
    return potx.ocr.DutyPaidProofOCR(img_path, img_url, configPath, id, key)


def EduPaperOCR(img_path=None, img_url=None, configPath=None, id=None, key=None):
    return potx.ocr.EduPaperOCR(img_path, img_url, configPath, id, key)


def EnglishOCR(img_path=None, img_url=None, configPath=None, id=None, key=None):
    return potx.ocr.EnglishOCR(img_path, img_url, configPath, id, key)


def EnterpriseLicenseOCR(img_path=None, img_url=None, configPath=None, id=None, key=None):
    return potx.ocr.EnterpriseLicenseOCR(img_path, img_url, configPath, id, key)


def EstateCertOCR(img_path=None, img_url=None, configPath=None, id=None, key=None):
    return potx.ocr.EstateCertOCR(img_path, img_url, configPath, id, key)


def FinanBillOCR(img_path=None, img_url=None, configPath=None, id=None, key=None):
    return potx.ocr.FinanBillOCR(img_path, img_url, configPath, id, key)


def FinanBillSliceOCR(img_path=None, img_url=None, configPath=None, id=None, key=None):
    return potx.ocr.FinanBillSliceOCR(img_path, img_url, configPath, id, key)


def FlightInvoiceOCR(img_path=None, img_url=None, configPath=None, id=None, key=None):
    return potx.ocr.FlightInvoiceOCR(img_path, img_url, configPath, id, key)


def FormulaOCR(img_path=None, img_url=None, configPath=None, id=None, key=None):
    return potx.ocr.FormulaOCR(img_path, img_url, configPath, id, key)


def GeneralAccurateOCR(img_path=None, img_url=None, configPath=None, id=None, key=None):
    return potx.ocr.GeneralAccurateOCR(img_path, img_url, configPath, id, key)


def GeneralBasicOCR(img_path=None, img_url=None, configPath=None, id=None, key=None):
    return potx.ocr.GeneralBasicOCR(img_path, img_url, configPath, id, key)


def GeneralEfficientOCR(img_path=None, img_url=None, configPath=None, id=None, key=None):
    return potx.ocr.GeneralEfficientOCR(img_path, img_url, configPath, id, key)


def GeneralFastOCR(img_path=None, img_url=None, configPath=None, id=None, key=None):
    return potx.ocr.GeneralFastOCR(img_path, img_url, configPath, id, key)


def GeneralHandwritingOCR(img_path=None, img_url=None, configPath=None, id=None, key=None):
    return potx.ocr.GeneralHandwritingOCR(img_path, img_url, configPath, id, key)


def HKIDCardOCR(img_path=None, img_url=None, configPath=None, id=None, key=None):
    return potx.ocr.HKIDCardOCR(img_path, img_url, configPath, id, key)


def HmtResidentPermitOCR(img_path=None, img_url=None, configPath=None, id=None, key=None):
    return potx.ocr.HmtResidentPermitOCR(img_path, img_url, configPath, id, key)


def IDCardOCR(img_path=None, img_url=None, configPath=None, id=None, key=None):
    return potx.ocr.IDCardOCR(img_path, img_url, configPath, id, key)


def ImageEnhancement(img_path=None, img_url=None, configPath=None, id=None, key=None):
    return potx.ocr.ImageEnhancement(img_path, img_url, configPath, id, key)


def InstitutionOCR(img_path=None, img_url=None, configPath=None, id=None, key=None):
    return potx.ocr.InstitutionOCR(img_path, img_url, configPath, id, key)


def InsuranceBillOCR(img_path=None, img_url=None, configPath=None, id=None, key=None):
    return potx.ocr.InsuranceBillOCR(img_path, img_url, configPath, id, key)


def InvoiceGeneralOCR(img_path=None, img_url=None, configPath=None, id=None, key=None):
    return potx.ocr.InvoiceGeneralOCR(img_path, img_url, configPath, id, key)


def LicensePlateOCR(img_path=None, img_url=None, configPath=None, id=None, key=None):
    return potx.ocr.LicensePlateOCR(img_path, img_url, configPath, id, key)


def MLIDCardOCR(img_path=None, img_url=None, configPath=None, id=None, key=None):
    return potx.ocr.MLIDCardOCR(img_path, img_url, configPath, id, key)


def MLIDPassportOCR(img_path=None, img_url=None, configPath=None, id=None, key=None):
    return potx.ocr.MLIDPassportOCR(img_path, img_url, configPath, id, key)


def MainlandPermitOCR(img_path=None, img_url=None, configPath=None, id=None, key=None):
    return potx.ocr.MainlandPermitOCR(img_path, img_url, configPath, id, key)


def MixedInvoiceDetect(img_path=None, img_url=None, configPath=None, id=None, key=None):
    return potx.ocr.MixedInvoiceDetect(img_path, img_url, configPath, id, key)


def MixedInvoiceOCR(img_path=None, img_url=None, configPath=None, id=None, key=None):
    return potx.ocr.MixedInvoiceOCR(img_path, img_url, configPath, id, key)


def OrgCodeCertOCR(img_path=None, img_url=None, configPath=None, id=None, key=None):
    return potx.ocr.OrgCodeCertOCR(img_path, img_url, configPath, id, key)


def PassportOCR(img_path=None, img_url=None, configPath=None, id=None, key=None):
    return potx.ocr.PassportOCR(img_path, img_url, configPath, id, key)


def PermitOCR(img_path=None, img_url=None, configPath=None, id=None, key=None):
    return potx.ocr.PermitOCR(img_path, img_url, configPath, id, key)


def PropOwnerCertOCR(img_path=None, img_url=None, configPath=None, id=None, key=None):
    return potx.ocr.PropOwnerCertOCR(img_path, img_url, configPath, id, key)


def QrcodeOCR(img_path=None, img_url=None, configPath=None, id=None, key=None):
    return potx.ocr.QrcodeOCR(img_path, img_url, configPath, id, key)


def QueryBarCode(img_path=None, img_url=None, configPath=None, id=None, key=None):
    return potx.ocr.QueryBarCode(img_path, img_url, configPath, id, key)


def QuotaInvoiceOCR(img_path=None, img_url=None, configPath=None, id=None, key=None):
    return potx.ocr.QuotaInvoiceOCR(img_path, img_url, configPath, id, key)


def RecognizeContainerOCR(img_path=None, img_url=None, configPath=None, id=None, key=None):
    return potx.ocr.RecognizeContainerOCR(img_path, img_url, configPath, id, key)


def RecognizeHealthCodeOCR(img_path=None, img_url=None, configPath=None, id=None, key=None):
    return potx.ocr.RecognizeHealthCodeOCR(img_path, img_url, configPath, id, key)


def RecognizeIndonesiaIDCardOCR(img_path=None, img_url=None, configPath=None, id=None, key=None):
    return potx.ocr.RecognizeIndonesiaIDCardOCR(img_path, img_url, configPath, id, key)


def RecognizeMedicalInvoiceOCR(img_path=None, img_url=None, configPath=None, id=None, key=None):
    return potx.ocr.RecognizeMedicalInvoiceOCR(img_path, img_url, configPath, id, key)


def RecognizeOnlineTaxiItineraryOCR(img_path=None, img_url=None, configPath=None, id=None, key=None):
    return potx.ocr.RecognizeOnlineTaxiItineraryOCR(img_path, img_url, configPath, id, key)


def RecognizePhilippinesDrivingLicenseOCR(img_path=None, img_url=None, configPath=None, id=None, key=None):
    return potx.ocr.RecognizePhilippinesDrivingLicenseOCR(img_path, img_url, configPath, id, key)


def RecognizePhilippinesVoteIDOCR(img_path=None, img_url=None, configPath=None, id=None, key=None):
    return potx.ocr.RecognizePhilippinesVoteIDOCR(img_path, img_url, configPath, id, key)


def RecognizeTableOCR(img_path=None, img_url=None, configPath=None, id=None, key=None):
    return potx.ocr.RecognizeTableOCR(img_path, img_url, configPath, id, key)


def RecognizeThaiIDCardOCR(img_path=None, img_url=None, configPath=None, id=None, key=None):
    return potx.ocr.RecognizeThaiIDCardOCR(img_path, img_url, configPath, id, key)


def RecognizeTravelCardOCR(img_path=None, img_url=None, configPath=None, id=None, key=None):
    return potx.ocr.RecognizeTravelCardOCR(img_path, img_url, configPath, id, key)


def ResidenceBookletOCR(img_path=None, img_url=None, configPath=None, id=None, key=None):
    return potx.ocr.ResidenceBookletOCR(img_path, img_url, configPath, id, key)


def RideHailingDriverLicenseOCR(img_path=None, img_url=None, configPath=None, id=None, key=None):
    return potx.ocr.RideHailingDriverLicenseOCR(img_path, img_url, configPath, id, key)


def RideHailingTransportLicenseOCR(img_path=None, img_url=None, configPath=None, id=None, key=None):
    return potx.ocr.RideHailingTransportLicenseOCR(img_path, img_url, configPath, id, key)


def SealOCR(img_path=None, img_url=None, configPath=None, id=None, key=None):
    return potx.ocr.SealOCR(img_path, img_url, configPath, id, key)


def ShipInvoiceOCR(img_path=None, img_url=None, configPath=None, id=None, key=None):
    return potx.ocr.ShipInvoiceOCR(img_path, img_url, configPath, id, key)


def SmartStructuralOCR(img_path=None, img_url=None, configPath=None, id=None, key=None):
    return potx.ocr.SmartStructuralOCR(img_path, img_url, configPath, id, key)


def TableOCR(img_path=None, img_url=None, configPath=None, id=None, key=None):
    return potx.ocr.TableOCR(img_path, img_url, configPath, id, key)


def TaxiInvoiceOCR(img_path=None, img_url=None, configPath=None, id=None, key=None):
    return potx.ocr.TaxiInvoiceOCR(img_path, img_url, configPath, id, key)


def TextDetect(img_path=None, img_url=None, configPath=None, id=None, key=None):
    return potx.ocr.TextDetect(img_path, img_url, configPath, id, key)


def TollInvoiceOCR(img_path=None, img_url=None, configPath=None, id=None, key=None):
    return potx.ocr.TollInvoiceOCR(img_path, img_url, configPath, id, key)


def TrainTicketOCR(img_path=None, img_url=None, configPath=None, id=None, key=None):
    return potx.ocr.TrainTicketOCR(img_path, img_url, configPath, id, key)


def VatInvoiceVerify(img_path=None, img_url=None, configPath=None, id=None, key=None):
    return potx.ocr.VatInvoiceVerify(img_path, img_url, configPath, id, key)


def VatInvoiceVerifyNew(img_path=None, img_url=None, configPath=None, id=None, key=None):
    return potx.ocr.VatInvoiceVerifyNew(img_path, img_url, configPath, id, key)


def VatRollInvoiceOCR(img_path=None, img_url=None, configPath=None, id=None, key=None):
    return potx.ocr.VatRollInvoiceOCR(img_path, img_url, configPath, id, key)


def VehicleLicenseOCR(img_path=None, img_url=None, configPath=None, id=None, key=None):
    return potx.ocr.VehicleLicenseOCR(img_path, img_url, configPath, id, key)


def VehicleRegCertOCR(img_path=None, img_url=None, configPath=None, id=None, key=None):
    return potx.ocr.VehicleRegCertOCR(img_path, img_url, configPath, id, key)


def VerifyBasicBizLicense(img_path=None, img_url=None, configPath=None, id=None, key=None):
    return potx.ocr.VerifyBasicBizLicense(img_path, img_url, configPath, id, key)


def VerifyBizLicense(img_path=None, img_url=None, configPath=None, id=None, key=None):
    return potx.ocr.VerifyBizLicense(img_path, img_url, configPath, id, key)


def VerifyEnterpriseFourFactors(img_path=None, img_url=None, configPath=None, id=None, key=None):
    return potx.ocr.VerifyEnterpriseFourFactors(img_path, img_url, configPath, id, key)


def VinOCR(img_path=None, img_url=None, configPath=None, id=None, key=None):
    return potx.ocr.VinOCR(img_path, img_url, configPath, id, key)


def WaybillOCR(img_path=None, img_url=None, configPath=None, id=None, key=None):
    return potx.ocr.WaybillOCR(img_path, img_url, configPath, id, key)


def RecognizeGeneralInvoice(img_path=None, img_url=None, configPath=None, id=None, key=None):
    return potx.ocr.RecognizeGeneralInvoice(img_path, img_url, configPath, id, key)


def social_security_card(img_path, id, key):
    baidu_ocr = BaiduOCR(id, key)
    return baidu_ocr.social_security_card(img_path)


# def VatInvoiceOCR(img_path=None, img_url=None, configPath=None, id=None, key=None):
#     """
#     增值税发票的识别
#     :param img_path: 必填，发票的图片位置
#     :param configPath: 选填，配置文件的位置，有默认值
#     :return:
#     """
#
#     ocr = get_ocr(configPath)
#     if img_url:
#         ocr_res = ocr.VatInvoiceOCR(ImageBase64=img_path, ImageUrl=img_url)
#     else:
#         ImageBase64 = img2base64(img_path)
#         ocr_res = ocr.VatInvoiceOCR(ImageBase64=ImageBase64, ImageUrl=img_url)
#     return ocr_res

def VatInvoiceOCR(img_path=None, img_url=None, configPath=None, id=None, key=None, pdf_path=None):
    return potx.ocr.VatInvoiceOCR(img_path, img_url, configPath, id, key, pdf_path)

    # return do_api(OCR_NAME=str(sys._getframe().f_code.co_name),
    #               img_path=img_path,
    #               img_url=img_url,
    #               configPath=configPath,
    #               id=id, key=key, pdf_path=pdf_path)


def VerifyOfdVatInvoiceOCR(ofd_file_path=None, ofd_file_url=None, id=None, key=None):
    return potx.ocr.VerifyOfdVatInvoiceOCR(ofd_file_path, ofd_file_url, id, key)
    # ocr = OfdOCR()
    # ocr.set_config(id, key)
    # if ofd_file_path:
    #     with open(ofd_file_path, 'rb') as file:
    #         pdf_data = file.read()
    #         base64_encoded_pdf = base64.b64encode(pdf_data).decode('utf-8')
    #         ocr_res = ocr.DoOCR(OfdFileBase64=base64_encoded_pdf)
    # elif ofd_file_url:
    #     ocr_res = ocr.DoOCR(OfdFileUrl=ofd_file_url)
    #
    # return ocr_res


def BankSlipOCR(img_path=None, img_url=None, configPath=None, id=None, key=None, pdf_path=None):
    return potx.ocr.BankSlipOCR(img_path, img_url, configPath, id, key, pdf_path)
    # return do_api(OCR_NAME=str(sys._getframe().f_code.co_name),
    #               img_path=img_path,
    #               img_url=img_url,
    #               configPath=configPath,
    #               id=id, key=key, pdf_path=pdf_path)


######## 优化示例函数 ########
def SmartStructuralPro(id, key, img_path):
    """
    直接调用potx中的方法，实现该功能
    Args:
        id:
        key:
        img_path:

    Returns:

    """
    return potx.ocr.SmartStructuralPro(id, key, img_path)



# -*- coding: UTF-8 -*-
'''
开通和使用👉[免费教程](https://www.bilibili.com/video/BV13J4m1s7L7/)
'''
import json
from collections import OrderedDict
from pathlib import Path

import pandas as pd
from loguru import logger
from pofile import get_files, mkdir
from poprogress import simple_progress

import poocr
from poocr.api.ocr import VatInvoiceOCR, IDCardOCR, BizLicenseOCR, BankSlipOCR, RecognizeGeneralInvoice, TrainTicketOCR
from poocr.core import HuaweiOCR

import potx


##### 调用poocr的原生接口 #####

def VatInvoiceOCR2Excel(input_path, output_path=r'./', output_excel='VatInvoiceOCR2Excel.xlsx', img_url=None,
                        configPath=None, id=None, key=None, file_name=False, trans=False):
    """
    将OCR识别的增值税发票数据转换为Excel表格。

    该函数主要处理从图像文件中提取的增值税发票数据，通过OCR技术识别后，将数据整理并输出到Excel表格中。
    这对于财务人员自动整理和核对发票信息非常有用。

    :param input_path: 输入文件路径，可以是单个文件或文件夹
    :param output_path: 输出Excel文件的路径，默认为None，表示使用函数默认文件名并保存在当前目录
    :param output_excel: 输出Excel文件的名称，默认为'VatInvoiceOCR2Excel.xlsx'
    :param img_url: 图像文件的URL地址，用于远程处理
    :param configPath: 配置文件路径，用于指定OCR引擎的配置
    :param id: OCR引擎的用户ID
    :param key: OCR引擎的用户密钥
    :param file_name: 是否在Excel中包含文件名作为一行数据，默认为False
    :param trans: 是否进行数据转换，默认为False。如果设置为True，将尝试将识别到的文本数据转换为相应的数字或日期格式
    """

    vat_img_files = get_files(input_path)
    if vat_img_files == None:
        raise BaseException(f'{input_path}这个文件目录下，没有存放任何发票，请确认后重新运行')
    abs_intput_path = Path(input_path).absolute()
    mkdir(Path(output_path).absolute())  # 如果不存在，则创建输出目录
    if output_excel.endswith('.xlsx') or output_excel.endswith('xls'):  # 如果指定的输出excel结尾不正确，则报错退出
        abs_output_excel = Path(output_path).absolute() / output_excel
    else:  # 指定了，但不是xlsx或者xls结束
        raise BaseException(
            f'输出结果名：output_excel参数，必须以xls或者xlsx结尾，您的输入:{output_excel}有误，请修改后重新运行')
    res_df = []  # 装全部识别的结果
    for vat_img in simple_progress(vat_img_files):
        try:

            if Path(vat_img).suffix == '.pdf':
                api_res = VatInvoiceOCR(pdf_path=str(vat_img), img_url=img_url, configPath=configPath, id=id, key=key)
            else:
                api_res = VatInvoiceOCR(img_path=str(vat_img), img_url=img_url, configPath=configPath, id=id, key=key)
                # 处理api_res可能是列表的情况（多页pdf）
            if isinstance(api_res, list):
                # 遍历每一页结果
                for page_res in api_res:
                    VatInvoiceOCR2(page_res, vat_img, file_name, trans, res_df)
            else:
                VatInvoiceOCR2(api_res, vat_img, file_name, trans, res_df)
        except Exception as e:
            logger.error(e)
            # continue
    # 整理全部识别结果
    if len(res_df) > 0:
        # 将所有结果转换位DataFrame并保存
        df = pd.DataFrame(res_df)
        df.to_excel(str(abs_output_excel), index=None, engine='openpyxl')
    else:
        logger.warning(f'该文件夹下，没有任何符合条件的发票图片/PDF文件')


def BankSlipOCR2Excel(input_path, output_path=r'./', output_excel='BankSlipOCR2Excel.xlsx', img_url=None,
                      configPath=None, id=None, key=None, file_name=False, trans=False):
    """
    将OCR识别的增值税发票数据转换为Excel表格。

    该函数主要处理从图像文件中提取的增值税发票数据，通过OCR技术识别后，将数据整理并输出到Excel表格中。
    这对于财务人员自动整理和核对发票信息非常有用。

    :param input_path: 输入文件路径，可以是单个文件或文件夹
    :param output_path: 输出Excel文件的路径，默认为None，表示使用函数默认文件名并保存在当前目录
    :param output_excel: 输出Excel文件的名称，默认为'BankSlipOCR2Excel.xlsx'
    :param img_url: 图像文件的URL地址，用于远程处理
    :param configPath: 配置文件路径，用于指定OCR引擎的配置
    :param id: OCR引擎的用户ID
    :param key: OCR引擎的用户密钥
    :param file_name: 是否在Excel中包含文件名作为一行数据，默认为False
    :param trans: 是否进行数据转换，默认为False。如果设置为True，将尝试将识别到的文本数据转换为相应的数字或日期格式
    """

    vat_img_files = get_files(input_path)
    if vat_img_files == None:
        raise BaseException(f'{input_path}这个文件目录下，没有存放任何发票，请确认后重新运行')
    abs_intput_path = Path(input_path).absolute()
    mkdir(Path(output_path).absolute())  # 如果不存在，则创建输出目录
    if output_excel.endswith('.xlsx') or output_excel.endswith('xls'):  # 如果指定的输出excel结尾不正确，则报错退出
        abs_output_excel = Path(output_path).absolute() / output_excel
    else:  # 指定了，但不是xlsx或者xls结束
        raise BaseException(
            f'输出结果名：output_excel参数，必须以xls或者xlsx结尾，您的输入:{output_excel}有误，请修改后重新运行')
    res_df = []  # 装全部识别的结果
    for vat_img in simple_progress(vat_img_files):
        try:
            if Path(vat_img).suffix == '.pdf':
                api_res = BankSlipOCR(img_path=str(vat_img), img_url=img_url, configPath=configPath, id=id, key=key)
            else:
                api_res = BankSlipOCR(img_path=str(vat_img), img_url=img_url, configPath=configPath, id=id, key=key)
            api_res_json = json.loads(str(api_res), object_pairs_hook=OrderedDict)
            bank_infos = api_res_json['BankSlipInfos']
            dict_pandas = {}  # 存放一行数据
            info_i = 0
            for info in bank_infos:
                info_i = info_i + 1
                name = info['Name']
                value = info['Value']
                if name == '机构' and info_i != 1:
                    d_p = pd.DataFrame(dict_pandas, index=[0])
                    res_df.append(d_p)
                    dict_pandas = {}  # 存放一行数据

                dict_pandas[name] = value
            if dict_pandas:
                d_p = pd.DataFrame(dict_pandas, index=[0])
                res_df.append(d_p)

        except Exception as e:
            logger.error(e)
            continue
    # 整理全部识别结果
    if len(res_df) > 0:
        res_excel = res_df[0]
        for index, line_df in enumerate(res_df):
            if index == 0:
                continue
            res_excel = res_excel._append(line_df)
        pd.DataFrame(res_excel).to_excel(str(abs_output_excel))  # 写入Excel
    else:
        logger.warning(f'该文件夹下，没有任何符合条件的发票图片/PDF文件')


def IDCardOCR2Excel(input_path, output_path=None, output_excel='IDCardOCR2Excel.xlsx', img_url=None,
                    configPath=None, id=None, key=None):
    """
    批量识别身份证，并保存在Excel中
    :param input_path: 身份证存放位置，可以填单个文件，也可以填一个目录
    :param output_path:
    :param output_excel:
    :param img_url:
    :param configPath:
    :return:
    """
    id_img_files = get_files(input_path)
    if id_img_files == None:
        raise BaseException(f'{input_path}这个路径下，没有存放任何身份证，请确认后重新运行')
    abs_intput_path = Path(input_path).absolute()
    if output_path == None:
        output_path = './'
    mkdir(Path(output_path).absolute())  # 如果不存在，则创建输出目录
    if output_excel.endswith('.xlsx') or output_excel.endswith('xls'):  # 如果指定的输出excel结尾不正确，则报错退出
        abs_output_excel = Path(output_path).absolute() / output_excel
    else:  # 指定了，但不是xlsx或者xls结束
        raise BaseException(
            f'输出结果名：output_excel参数，必须以xls或者xlsx结尾，您的输入:{output_excel}有误，请修改后重新运行')
    res_df = []  # 装全部识别的结果
    for id_img in simple_progress(id_img_files):
        try:
            api_res = IDCardOCR(img_path=str(id_img), img_url=img_url, configPath=configPath, id=id, key=key)
            api_res_json = json.loads(str(api_res))
            del api_res_json['ReflectDetailInfos']

            res_df.append(pd.DataFrame(api_res_json, index=[0]))
        except:
            continue
    # 整理全部识别结果
    if len(res_df) > 0:
        res_excel = res_df[0]
        for index, line_df in enumerate(res_df):
            if index == 0:
                continue
            res_excel = res_excel._append(line_df)
        pd.DataFrame(res_excel).to_excel(str(abs_output_excel))  # 写入Excel
    else:
        logger.info(f'该文件夹下，没有任何符合条件的身份证图片')


def TrainTicketOCR2Excel(input_path: str, output_excel: str = r'./TrainTicketOCR2Excel.xlsx', img_url: str = None,
                         configPath: str = None, id=None, key=None) -> None:
    ticket_list = []
    ticket_files = get_files(input_path)
    for ticket in simple_progress(ticket_files):
        try:
            api_res = TrainTicketOCR(img_path=ticket, img_url=img_url, configPath=configPath,id=id,key=key)
            api_res_json = json.loads(str(api_res))
            ticket_list.append(pd.DataFrame(api_res_json, index=[0]))
        except:
            continue
    # 整理全部识别结果
    if len(ticket_list) > 0:
        res_excel = ticket_list[0]
        for index, line_df in enumerate(ticket_list):
            if index == 0:
                continue
            res_excel = res_excel._append(line_df)
        pd.DataFrame(res_excel).to_excel(str(output_excel))  # 写入Excel
    else:
        logger.info(f'该文件夹下，没有任何符合条件的火车票图片')


def BankCardOCR2Excel(input_path, output_path=None, output_excel='BankCardOCR2Excel.xlsx', img_url=None,
                      configPath=None, id=None, key=None):
    """
    识别银行卡，自动保存为Excel文件
    :param input_path: 必填，银行卡图片的位置
    :param output_path: 选填，输出Excel的位置
    :param output_excel: 选填，输出Excel的名称
    :param img_url: 选填，可以是网络图片
    :param configPath: 已废弃
    :param id: 你的腾讯账号的密钥，获取方式：https://curl.qcloud.com/fuOGcm2R
    :param key: 你的腾讯账号的密钥，获取方式：https://curl.qcloud.com/fuOGcm2R
    :return:
    """
    test_json = poocr.ocr.BankCardOCR(img_path=input_path, img_url=img_url, configPath=configPath, id=id,
                                      key=key)
    df = pd.DataFrame(json.loads(str(test_json)), index=[0])
    if output_path == None:
        output_path = './'
    mkdir(Path(output_path).absolute())  # 如果不存在，则创建输出目录
    if output_excel.endswith('.xlsx') or output_excel.endswith('xls'):  # 如果指定的输出excel结尾不正确，则报错退出
        abs_output_excel = Path(output_path).absolute() / output_excel
    df.to_excel(str(abs_output_excel), index=False)


def LicensePlateOCR2Excel(input_path, output_path=None, output_excel='LicensePlateOCR2Excel.xlsx', img_url=None,
                          configPath=None, id=None, key=None):
    """
    识别银行卡，自动保存为Excel文件
    :param input_path: 必填，银行卡图片的位置
    :param output_path: 选填，输出Excel的位置
    :param output_excel: 选填，输出Excel的名称
    :param img_url: 选填，可以是网络图片
    :param configPath: 已废弃
    :param id: 你的腾讯账号的密钥，获取方式：https://curl.qcloud.com/fuOGcm2R
    :param key: 你的腾讯账号的密钥，获取方式：https://curl.qcloud.com/fuOGcm2R
    :return:
    """
    test_json = poocr.ocr.LicensePlateOCR(img_path=input_path, img_url=img_url, configPath=configPath, id=id,
                                          key=key)
    df = pd.DataFrame(json.loads(str(test_json)), index=[0])
    if output_path == None:
        output_path = './'
    mkdir(Path(output_path).absolute())  # 如果不存在，则创建输出目录
    if output_excel.endswith('.xlsx') or output_excel.endswith('xls'):  # 如果指定的输出excel结尾不正确，则报错退出
        abs_output_excel = Path(output_path).absolute() / output_excel
    df.to_excel(str(abs_output_excel), index=False)


def BizLicenseOCR2Excel(input_path, output_path=r'./', output_excel='BizLicenseOCR2Excel.xlsx', img_url=None,
                        configPath=None, id=None, key=None, file_name=False, trans=False):
    """
    将营业执照OCR识别结果整理并保存到Excel中。

    :param input_path: 图片输入路径，包含营业执照图片。
    :param output_path: 输出路径，默认为当前目录。
    :param output_excel: 输出的Excel文件名，默认为'BizLicenseOCR2Excel.xlsx'。
    :param img_url: 图片URL，如果提供，将通过URL进行识别。
    :param configPath: 配置文件路径，用于指定OCR识别的配置。
    :param id: API的用户ID。
    :param key: API的密钥。
    :param file_name: 是否在结果中包含文件名，默认为False。
    :param trans: 是否进行翻译，默认为False。
    :raises BaseException: 当输入目录为空或输出文件名格式不正确时抛出异常。
    """
    # 获取输入路径下的所有文件
    vat_img_files = get_files(input_path)
    # 如果文件列表为空，抛出异常
    if vat_img_files == None:
        raise BaseException(f'{input_path}这个文件目录下，没有存放任何营业执照，请确认后重新运行')
    # 获取输入路径的绝对路径
    abs_intput_path = Path(input_path).absolute()
    # 创建输出路径
    mkdir(Path(output_path).absolute())  # 如果不存在，则创建输出目录
    # 检查输出文件名是否以.xlsx或.xls结尾，如果不是，抛出异常
    if output_excel.endswith('.xlsx') or output_excel.endswith('xls'):  # 如果指定的输出excel结尾不正确，则报错退出
        abs_output_excel = Path(output_path).absolute() / output_excel
    else:  # 指定了，但不是xlsx或者xls结束
        raise BaseException(
            f'输出结果名：output_excel参数，必须以xls或者xlsx结尾，您的输入:{output_excel}有误，请修改后重新运行')
    # 初始化结果数据框
    res_df = []  # 装全部识别的结果
    # 对每个营业执照图片进行识别
    for vat_img in simple_progress(vat_img_files):
        try:
            # 调用OCR API进行识别
            api_res = BizLicenseOCR(img_path=str(vat_img), img_url=img_url, configPath=configPath, id=id, key=key)
            # 将识别结果转换为JSON格式并添加到结果数据框
            api_res_json = json.loads(str(api_res))
            res_df.append(api_res_json)
        except Exception as e:
            # 打印识别失败的信息
            logger.info(f'{vat_img}识别失败，原因：{e}')
    # 将所有识别结果合并成一个数据框
    biz_def = pd.DataFrame(res_df)
    # 将结果数据框保存到Excel文件
    biz_def.to_excel(str(abs_output_excel), index=None)


def RecognizeGeneralInvoiceOCR2Excel(input_path, output_path=None, output_excel='RecognizeGeneralInvoiceOCR2Excel.xlsx',
                                     img_url=None, configPath=None, id=None, key=None, sub_type=None):
    """
   通用识别发票信息并保存到Excel中。

   :param input_path: 图片输入路径。
   :param output_path: 输出路径，默认为当前目录。
   :param output_excel: 输出的Excel文件名，默认为'BizLicenseOCR2Excel.xlsx'。
   :param img_url: 图片URL，如果提供，将通过URL进行识别。
   :param configPath: 配置文件路径，用于指定OCR识别的配置。
   :param id: API的用户ID。
   :param key: API的密钥。
   :param sub_type: 识别的类型 具体参数详见:https://cloud.tencent.com/document/api/866/90802  中SubType的列表信息
   :raises BaseException: 当输入目录为空或输出文件名格式不正确时抛出异常。
   """
    # 获取输入路径下的所有文件
    img_paths = get_files(input_path)
    if img_paths is None or len(img_paths) == 0:
        raise BaseException(f'{input_path}目录下,无机动车发票,请检查后重新运行.')
    if output_path is None:
        output_path = './'
    mkdir(Path(output_path).absolute())  # 如果不存在，则创建输出目录
    if output_excel.endswith('.xlsx') or output_excel.endswith('xls'):  # 如果指定的输出excel结尾不正确，则报错退出
        abs_output_excel = Path(output_path).absolute() / output_excel
    else:  # 指定了，但不是xlsx或者xls结束
        raise BaseException(
            f'输出结果名：output_excel参数，必须以xls或者xlsx结尾，您的输入:{output_excel}有误，请修改后重新运行')
    res_df = []
    try:
        for item in simple_progress(img_paths):

            api_res = RecognizeGeneralInvoice(img_path=str(item), img_url=img_url, configPath=configPath, id=id,
                                              key=key)
            api_res_json = json.loads(str(api_res))
            invoice_items = api_res_json['MixedInvoiceItems']
            for invoice_item in invoice_items:
                invoice_info = {}
                if sub_type is not None and invoice_item['Type'] != sub_type:
                    continue
                if invoice_item['Code'] == 'OK':
                    single_invoice_info = invoice_item['SingleInvoiceInfos']
                    for item in single_invoice_info[invoice_item['SubType']]:
                        if isinstance(single_invoice_info[invoice_item['SubType']][item], (str, int)):
                            invoice_info[item] = single_invoice_info[invoice_item['SubType']][item]
                res_df.append(invoice_info)
    except Exception as e:
        logger.info(f'{item}识别失败，原因：{e}')
    biz_def = pd.DataFrame(res_df)
    biz_def.to_excel(str(abs_output_excel), index=None)


def VatInvoiceOCR2(api_res, vat_img, file_name, trans, res_df):
    '''
    处理单个结果
    Args:
        api_res: 识别结果
        vat_img: 图片路径
        file_name: 是否包含文件铭
        trans: 是否进行数据转换
        res_df: 结果列表，用于村粗处理后的数据

    Returns:

    '''
    try:
        api_res_json = json.loads(str(api_res))
        # 处理VatInvoiceInfos部分
        if 'VatInvoiceInfos' in api_res_json:
            VatInvoiceInfos = api_res_json['VatInvoiceInfos']
            dict_pandas = {}

            # add文件铭
            if file_name:
                dict_pandas['文件名'] = Path(vat_img).name

            # 处理备注字段
            beizhu_value = ''

            # 处理所有发票
            for item in VatInvoiceInfos:
                if item['Name'] == '备注':
                    beizhu_value += item['Value']
                else:
                    if trans:
                        pass
                    else:
                        dict_pandas[item['Name']] = item['Value']
            # 添加备注字段
            dict_pandas['备注'] = beizhu_value

            # 处理Items部分
            if 'Items' in api_res_json:
                field_mapping = {
                    'Name': '商品名称',
                    'Spec': '规格型号',
                    'Unit': '单位',
                    'Quantity': '数量',
                    'UnitPrice': '单价',
                    'AmountWithoutTax': '金额'}

                Items = api_res_json['Items']
                if Items and len(Items) > 0:
                    dict_pandas['有商品明细'] = '是'

                    # 将第一个商品信息添加到结果
                    first_item = Items[0]
                    for key, value in first_item.items():
                        column_name = f"{field_mapping.get(key, key)}"
                        dict_pandas[column_name] = value
                else:
                    dict_pandas['有商品明细'] = '否'
            # 将处理好的结果添加到结果列表
            res_df.append(dict_pandas)

            logger.info(f"成功处理文件: {vat_img}")

    except Exception as e:
        logger.error(f"处理文件 {vat_img} 时出错: {str(e)}")


##### 调用腾讯的接口 #####
def RET2excel(img_path, id, key, output_path=r'./', output_excel='RET2excel.xlsx'):
    potx.ocr2excel.RET2excel(img_path=img_path, id=id, key=key, output_path=output_path, output_excel=output_excel)


##### 调用华为的接口 #####
def household2excel(ak, sk, img_path, output_excel='household2excel.xlsx'):
    """
    将户口图片转换为Excel格式。

    本函数通过调用华为云的OCR服务，将户口图片中的信息识别并转换为Excel格式。
    使用华为云账号的访问密钥（ak）和安全密钥（sk）进行身份验证，
    指定图片文件的路径（img_path），并可选地指定输出的Excel文件名（output_excel）。

    参数:
    ak (str): 华为云账号的访问密钥。
    sk (str): 华为云账号的安全密钥。
    img_path (str): 户口图片文件的路径。
    output_excel (str, 可选): 输出的Excel文件名。默认为'household2excel.xlsx'。

    返回:
    无直接返回值，结果为指定路径的Excel文件。
    """
    HuaweiOCR.household2excel(ak, sk, img_path, output_excel)


def household2excel2(ak, sk, img_path, output_excel='household2excel.xlsx'):
    """
    将户口图像文件转换为Excel格式。

    本函数通过调用华为OCR服务，将户口图像文件中的信息识别并提取出来，然后将这些信息保存到Excel文件中。
    使用华为云的AK和SK进行身份验证，指定图像文件路径进行识别，可选地指定输出的Excel文件名。

    参数:
    - ak: 华为云的Access Key，用于身份验证。
    - sk: 华为云的Secret Key，用于身份验证。
    - img_path: 户口图像文件的路径，用于识别。
    - output_excel: 输出的Excel文件名，默认为'household2excel.xlsx'。

    返回:
    无返回值，直接将识别结果保存在Excel文件中。
    """
    HuaweiOCR.household2excel2(ak, sk, img_path, output_excel)
